# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: attestation.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x61ttestation.proto\x12\x0b\x61ttestation\"\xc2\x01\n\x08\x46\x61tquote\x12)\n\x04\x65pid\x18\x01 \x01(\x0b\x32\x19.attestation.FatquoteEpidH\x00\x12)\n\x04\x64\x63\x61p\x18\x02 \x01(\x0b\x32\x19.attestation.FatquoteDcapH\x00\x12+\n\x05nitro\x18\x03 \x01(\x0b\x32\x1a.attestation.FatquoteNitroH\x00\x12\'\n\x03snp\x18\x04 \x01(\x0b\x32\x18.attestation.FatquoteSnpH\x00\x42\n\n\x08\x66\x61tquote\"k\n\x0c\x46\x61tquoteEpid\x12\x17\n\x0fiasResponseBody\x18\x01 \x01(\x0c\x12\x16\n\x0eiasCertificate\x18\x02 \x01(\x0c\x12\x14\n\x0ciasSignature\x18\x03 \x01(\x0c\x12\x14\n\x0ciasRootCaDer\x18\x04 \x01(\x0c\"\x86\x01\n\x0c\x46\x61tquoteDcap\x12\x11\n\tdcapQuote\x18\x01 \x01(\x0c\x12\x0f\n\x07tcbInfo\x18\x02 \x01(\x0c\x12\x12\n\nqeIdentity\x18\x03 \x01(\x0c\x12\x13\n\x0btcbSignCert\x18\x04 \x01(\x0c\x12\x12\n\nqeSignCert\x18\x05 \x01(\x0c\x12\x15\n\rdcapRootCaDer\x18\x06 \x01(\x0c\"5\n\rFatquoteNitro\x12\x0c\n\x04\x63ose\x18\x01 \x01(\x0c\x12\x16\n\x0enitroRootCaDer\x18\x02 \x01(\x0c\"\xdf\x01\n\x0b\x46\x61tquoteSnp\x12\x11\n\treportBin\x18\x01 \x01(\x0c\x12\x11\n\tamdArkDer\x18\x02 \x01(\x0c\x12\x11\n\tamdSevDer\x18\x03 \x01(\x0c\x12\x12\n\nvcekCrtDer\x18\x04 \x01(\x0c\x12\x12\n\nreportData\x18\x05 \x01(\x0c\x12\x17\n\x0froughtimePubKey\x18\x06 \x01(\x0c\x12\x16\n\x0eroughtimeNonce\x18\x07 \x01(\x0c\x12\x17\n\x0fsignedTimestamp\x18\x08 \x01(\x0c\x12\x14\n\x0c\x64\x65\x63\x65ntriqDer\x18\t \x01(\x0c\x12\x0f\n\x07\x63hipDer\x18\n \x01(\x0c\"\xc3\x02\n\x18\x41ttestationSpecification\x12\x43\n\tintelEpid\x18\x01 \x01(\x0b\x32..attestation.AttestationSpecificationIntelEpidH\x00\x12\x43\n\tintelDcap\x18\x02 \x01(\x0b\x32..attestation.AttestationSpecificationIntelDcapH\x00\x12\x41\n\x08\x61wsNitro\x18\x03 \x01(\x0b\x32-.attestation.AttestationSpecificationAwsNitroH\x00\x12=\n\x06\x61mdSnp\x18\x04 \x01(\x0b\x32+.attestation.AttestationSpecificationAmdSnpH\x00\x42\x1b\n\x19\x61ttestation_specification\"\xa9\x01\n!AttestationSpecificationIntelEpid\x12\x11\n\tmrenclave\x18\x01 \x01(\x0c\x12\x14\n\x0ciasRootCaDer\x18\x02 \x01(\x0c\x12\x14\n\x0c\x61\x63\x63\x65pt_debug\x18\x03 \x01(\x08\x12 \n\x18\x61\x63\x63\x65pt_group_out_of_date\x18\x04 \x01(\x08\x12#\n\x1b\x61\x63\x63\x65pt_configuration_needed\x18\x05 \x01(\x08\"\xbc\x01\n!AttestationSpecificationIntelDcap\x12\x11\n\tmrenclave\x18\x01 \x01(\x0c\x12\x15\n\rdcapRootCaDer\x18\x02 \x01(\x0c\x12\x14\n\x0c\x61\x63\x63\x65pt_debug\x18\x03 \x01(\x08\x12\x1a\n\x12\x61\x63\x63\x65pt_out_of_date\x18\x04 \x01(\x08\x12#\n\x1b\x61\x63\x63\x65pt_configuration_needed\x18\x05 \x01(\x08\x12\x16\n\x0e\x61\x63\x63\x65pt_revoked\x18\x06 \x01(\x08\"r\n AttestationSpecificationAwsNitro\x12\x16\n\x0enitroRootCaDer\x18\x01 \x01(\x0c\x12\x0c\n\x04pcr0\x18\x02 \x01(\x0c\x12\x0c\n\x04pcr1\x18\x03 \x01(\x0c\x12\x0c\n\x04pcr2\x18\x04 \x01(\x0c\x12\x0c\n\x04pcr8\x18\x05 \x01(\x0c\"\x92\x01\n\x1e\x41ttestationSpecificationAmdSnp\x12\x11\n\tamdArkDer\x18\x01 \x01(\x0c\x12\x13\n\x0bmeasurement\x18\x02 \x01(\x0c\x12\x17\n\x0froughtimePubKey\x18\x03 \x01(\x0c\x12\x19\n\x11\x61uthorizedChipIds\x18\x04 \x03(\x0c\x12\x14\n\x0c\x64\x65\x63\x65ntriqDer\x18\x05 \x01(\x0c\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'attestation_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _FATQUOTE._serialized_start=35
  _FATQUOTE._serialized_end=229
  _FATQUOTEEPID._serialized_start=231
  _FATQUOTEEPID._serialized_end=338
  _FATQUOTEDCAP._serialized_start=341
  _FATQUOTEDCAP._serialized_end=475
  _FATQUOTENITRO._serialized_start=477
  _FATQUOTENITRO._serialized_end=530
  _FATQUOTESNP._serialized_start=533
  _FATQUOTESNP._serialized_end=756
  _ATTESTATIONSPECIFICATION._serialized_start=759
  _ATTESTATIONSPECIFICATION._serialized_end=1082
  _ATTESTATIONSPECIFICATIONINTELEPID._serialized_start=1085
  _ATTESTATIONSPECIFICATIONINTELEPID._serialized_end=1254
  _ATTESTATIONSPECIFICATIONINTELDCAP._serialized_start=1257
  _ATTESTATIONSPECIFICATIONINTELDCAP._serialized_end=1445
  _ATTESTATIONSPECIFICATIONAWSNITRO._serialized_start=1447
  _ATTESTATIONSPECIFICATIONAWSNITRO._serialized_end=1561
  _ATTESTATIONSPECIFICATIONAMDSNP._serialized_start=1564
  _ATTESTATIONSPECIFICATIONAMDSNP._serialized_end=1710
# @@protoc_insertion_point(module_scope)



from typing import List, Any, Optional
import uuid


__all__ = [ "Node" ]


class Node:
    name: str
    """The name of the node."""

    config: bytes
    """Serialized configuration to use in the compute node definition."""

    enclave_type: str
    """What type of enclave this node uses."""

    dependencies: List[str]
    """A list of names of nodes on which this node directly depends."""

    output_format: Any
    """The `proto.ComputeNodeFormat` of the output from this node."""

    def __init__(
            self,
            name: str,
            config: bytes,
            enclave_type: str,
            dependencies: List[str],
            output_format: Any
    ):
        self.name = name
        self.config = config
        self.enclave_type = enclave_type
        self.dependencies = dependencies
        self.output_format = output_format



from .compute_sql_pb2 import (
    PrimitiveType,
    TableSchema,
    NamedColumn,
    Constraint,
    PrivacySettings,
    ComputationConfiguration,
    ValidationConfiguration,
    ColumnType,
    SqlWorkerConfiguration,
    TableDependencyMapping
)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: google_dv_360_sink.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x18google_dv_360_sink.proto\x12\x12google_dv_360_sink\"\x1a\n\nSingleFile\x12\x0c\n\x04name\x18\x01 \x01(\t\"\t\n\x07RawFile\"L\n\x07ZipFile\x12\x34\n\nsingleFile\x18\x01 \x01(\x0b\x32\x1e.google_dv_360_sink.SingleFileH\x00\x42\x0b\n\tselection\"\x8d\x01\n\tSinkInput\x12\x12\n\ndependency\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12*\n\x03raw\x18\x03 \x01(\x0b\x32\x1b.google_dv_360_sink.RawFileH\x00\x12*\n\x03zip\x18\x04 \x01(\x0b\x32\x1b.google_dv_360_sink.ZipFileH\x00\x42\x06\n\x04\x66ile\"\xd1\x01\n\"GoogleDv360SinkWorkerConfiguration\x12,\n\x05input\x18\x01 \x01(\x0b\x32\x1d.google_dv_360_sink.SinkInput\x12\x1d\n\x15\x63redentialsDependency\x18\x02 \x01(\t\x12\x14\n\x0c\x61\x64vertiserId\x18\x03 \x01(\t\x12\x13\n\x0b\x64isplayName\x18\x04 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x05 \x01(\t\x12\x1e\n\x16membershipDurationDays\x18\x06 \x01(\tb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'google_dv_360_sink_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SINGLEFILE._serialized_start=48
  _SINGLEFILE._serialized_end=74
  _RAWFILE._serialized_start=76
  _RAWFILE._serialized_end=85
  _ZIPFILE._serialized_start=87
  _ZIPFILE._serialized_end=163
  _SINKINPUT._serialized_start=166
  _SINKINPUT._serialized_end=307
  _GOOGLEDV360SINKWORKERCONFIGURATION._serialized_start=310
  _GOOGLEDV360SINKWORKERCONFIGURATION._serialized_end=519
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: synth_data.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import compute_sql_pb2 as compute__sql__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x10synth_data.proto\x12\nsynth_data\x1a\x11\x63ompute_sql.proto\"j\n\x11SyntheticDataConf\x12#\n\x07\x63olumns\x18\x01 \x03(\x0b\x32\x12.synth_data.Column\x12\x1f\n\x17outputOriginalDataStats\x18\x02 \x01(\x08\x12\x0f\n\x07\x65psilon\x18\x03 \x01(\x02\"\xeb\x01\n\x04Mask\x12+\n\x06\x66ormat\x18\x01 \x01(\x0e\x32\x1b.synth_data.Mask.MaskFormat\"\xb5\x01\n\nMaskFormat\x12\x12\n\x0eGENERIC_STRING\x10\x00\x12\x12\n\x0eGENERIC_NUMBER\x10\x01\x12\x08\n\x04NAME\x10\x02\x12\x0b\n\x07\x41\x44\x44RESS\x10\x03\x12\x0c\n\x08POSTCODE\x10\x04\x12\x10\n\x0cPHONE_NUMBER\x10\x05\x12\x1a\n\x16SOCIAL_SECURITY_NUMBER\x10\x06\x12\t\n\x05\x45MAIL\x10\x07\x12\x08\n\x04\x44\x41TE\x10\x08\x12\r\n\tTIMESTAMP\x10\t\x12\x08\n\x04IBAN\x10\n\"^\n\x06\x43olumn\x12\r\n\x05index\x18\x01 \x01(\x05\x12%\n\x04type\x18\x02 \x01(\x0b\x32\x17.compute_sql.ColumnType\x12\x1e\n\x04mask\x18\x03 \x01(\x0b\x32\x10.synth_data.Maskb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'synth_data_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _SYNTHETICDATACONF._serialized_start=51
  _SYNTHETICDATACONF._serialized_end=157
  _MASK._serialized_start=160
  _MASK._serialized_end=395
  _MASK_MASKFORMAT._serialized_start=214
  _MASK_MASKFORMAT._serialized_end=395
  _COLUMN._serialized_start=397
  _COLUMN._serialized_end=491
# @@protoc_insertion_point(module_scope)



from .data_room_pb2 import (
    Permission,
    LeafCrudPermission,
    RetrieveAuditLogPermission,
    ExecuteComputePermission,
    RetrieveComputeResultPermission,
    RetrieveDataRoomPermission,
    RetrieveDataRoomStatusPermission,
    GenerateMergeSignaturePermission,
    UpdateDataRoomStatusPermission,
    RetrievePublishedDatasetsPermission,
    ExecuteDevelopmentComputePermission,
    MergeConfigurationCommitPermission,
    DryRunPermission,
    AuthenticationMethod,
    UserPermission,
    ComputeNode,
    ComputeNodeLeaf,
    ComputeNodeBranch,
    DataRoom,
    ComputeNodeParameter,
    ComputeNodeFormat,
    ComputeNodeProtocol,
    ConfigurationModification,
    AddModification,
    ChangeModification,
    ConfigurationElement,
    GovernanceProtocol,
    StaticDataRoomPolicy,
    AffectedDataOwnersApprovePolicy,
    ConfigurationCommit,
    DataRoomConfiguration,
    PkiPolicy,
    DcrSecretPolicy,
    ComputeNodeAirlock,
)
from .attestation_pb2 import (
    AttestationSpecification,
    AttestationSpecificationIntelEpid,
    Fatquote,
    FatquoteEpid,
    FatquoteDcap,
    AttestationSpecificationIntelDcap,
    AttestationSpecificationAwsNitro,
    AttestationSpecificationAmdSnp,
)
from .gcg_pb2 import (
    DriverTaskConfig,
    NoopConfig,
    DataRoomStatus,
    CreateDataRoomRequest,
    CreateDataRoomResponse,
    ExecuteComputeRequest,
    ExecuteComputeResponse,
    GcgRequest,
    GcgResponse,
    GetResultsRequest,
    GetResultsResponseChunk,
    JobStatusRequest,
    JobStatusResponse,
    PublishDatasetToDataRoomRequest,
    PublishDatasetToDataRoomResponse,
    RemovePublishedDatasetRequest,
    RemovePublishedDatasetResponse,
    RetrieveAuditLogRequest,
    RetrieveAuditLogResponse,
    RetrieveDataRoomRequest,
    RetrieveDataRoomResponse,
    RetrieveDataRoomStatusRequest,
    RetrieveDataRoomStatusResponse,
    RetrievePublishedDatasetsRequest,
    RetrievePublishedDatasetsResponse,
    UpdateDataRoomStatusRequest,
    ExecuteDevelopmentComputeRequest,
    CreateConfigurationCommitRequest,
    GenerateMergeApprovalSignatureRequest,
    MergeConfigurationCommitRequest,
    CreateConfigurationCommitResponse,
    GenerateMergeApprovalSignatureResponse,
    MergeConfigurationCommitResponse,
    UpdateDataRoomStatusResponse,
    RetrieveCurrentDataRoomConfigurationRequest,
    RetrieveCurrentDataRoomConfigurationResponse,
    RetrieveConfigurationCommitRequest,
    RetrieveConfigurationCommitResponse,
    StaticContentConfig,
    CreateConfigurationCommitRequest,
    RetrieveConfigurationCommitApproversRequest,
    UserAuth,
    Pki,
    TestDataset,
    RetrieveUsedAirlockQuotaRequest,
    RetrieveUsedAirlockQuotaResponse,
)
from .length_delimited import parse_length_delimited, serialize_length_delimited
from .delta_enclave_api_pb2 import (
    ChunkHeader,
    EncryptionHeader,
    VersionHeader,
    DataNoncePubkey,
    Request,
    Response,
)
from .synth_data_pb2 import (
    SyntheticDataConf,
    Mask,
    Column,
)
from .metering_pb2 import DcrMetadata, CreateDcrPurpose, CreateDcrKind
from .identity_endorsement_pb2 import (
    DcrSecretEndorsementRequest,
    DcrSecretEndorsementResponse,
    EndorsementRequest,
    EndorsementResponse,
    PkiEndorsementRequest,
    PkiEndorsementResponse,
    EnclaveEndorsement,
    EnclaveEndorsements,
)



from __future__ import annotations
from google.protobuf.json_format import MessageToDict
from typing import List, Optional, Set, Tuple
from .proto import (
    PrimitiveType, PrivacySettings, ComputationConfiguration, TableSchema,
    ValidationConfiguration, Constraint, NamedColumn, ColumnType, SqlWorkerConfiguration,
    TableDependencyMapping
)
from ..proto import serialize_length_delimited, ComputeNodeFormat, parse_length_delimited
from ..node import Node


class SqlCompute(Node):
    """
    Computation node to execute an SQL query.
    """

    def __init__(
            self,
            name: str,
            sql_statement: str,
            dependencies: List[Tuple[str, str]],
            *,
            privacy_settings: Optional[PrivacySettings] = None,
            constraints: Optional[List[Constraint]] = None,
    ) -> None:
        sql_worker_configuration = SqlWorkerConfiguration(
            computation=ComputationConfiguration(
                sqlStatement=sql_statement,
                privacySettings=privacy_settings,
                constraints=constraints,
                tableDependencyMappings=[
                    TableDependencyMapping(table=table, dependency=node_id)
                    for table, node_id in dependencies
                ]
            )
        )
        config = serialize_length_delimited(sql_worker_configuration)

        super().__init__(
            name,
            config=config,
            enclave_type="decentriq.sql-worker",
            dependencies=[node_id for _, node_id in dependencies],
            output_format=ComputeNodeFormat.ZIP
        )


class SqlSchemaVerifier(Node):
    """
    Computation node to validate an input and provide the necessary types.
    """

    def __init__(
        self,
        name: str,
        input_data_node: str,
        columns, # type: List[Tuple[str, PrimitiveType.V, bool]]
    ) -> None:
        named_columns = map(lambda c: NamedColumn(
            name=c[0],
            columnType=ColumnType(
                primitiveType=c[1],
                nullable=c[2]
                )
            ),
            columns
        )
        sql_worker_configuration = SqlWorkerConfiguration(
            validation=ValidationConfiguration(
                tableSchema=TableSchema(namedColumns=named_columns)
            )
        )
        config = serialize_length_delimited(sql_worker_configuration)

        super().__init__(
            name,
            config=config,
            enclave_type="decentriq.sql-worker",
            dependencies=[input_data_node],
            output_format=ComputeNodeFormat.ZIP
        )


class SqlWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = SqlWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)



from google.protobuf.json_format import MessageToDict
from ..proto import serialize_length_delimited, ComputeNodeFormat, parse_length_delimited
from ..node import Node
from typing import List, Optional, Set, Tuple
from .proto import (
    DataSourceSnowflakeWorkerConfiguration,
    SnowflakeSource,
)
from ..sql.proto import TableSchema, NamedColumn, ColumnType

__docformat__ = "restructuredtext"
__pdoc__ = {
    "proto": False,
}

class DataSourceSnowflake(Node):
    """
    Compute node that fetches a dataset from a Snowflake stage.
    """

    def __init__(
            self,
            name: str,
            warehouseName: str,
            databaseName: str,
            schemaName: str,
            tableName: str,
            stageName: str,
            credentials_dependency: str,
    ) -> None:
        config = DataSourceSnowflakeWorkerConfiguration(
            source=SnowflakeSource(
                warehouseName=warehouseName,
                databaseName=databaseName,
                schemaName=schemaName,
                tableName=tableName,
                stageName=stageName,
            ),
            credentialsDependency=credentials_dependency,
        )
        config_serialized = serialize_length_delimited(config)
        super().__init__(
            name,
            config=config_serialized,
            enclave_type="decentriq.data-source-snowflake-worker",
            dependencies=[credentials_dependency],
            output_format=ComputeNodeFormat.ZIP
        )

class DataSourceSnowflakeWorkerDecoder:
    def decode(self, config: bytes):
        config_decoded = DataSourceSnowflakeWorkerConfiguration()
        parse_length_delimited(config, config_decoded)
        return MessageToDict(config_decoded)

__all__ = [
    "DataSourceSnowflake",
]



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 8):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing_extensions.final
class ContainerWorkerConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    STATIC_FIELD_NUMBER: builtins.int
    @property
    def static(self) -> global___StaticImage: ...
    def __init__(
        self,
        *,
        static: global___StaticImage | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["configuration", b"configuration", "static", b"static"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["configuration", b"configuration", "static", b"static"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["configuration", b"configuration"]) -> typing_extensions.Literal["static"] | None: ...

global___ContainerWorkerConfiguration = ContainerWorkerConfiguration

@typing_extensions.final
class StaticImage(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    COMMAND_FIELD_NUMBER: builtins.int
    MOUNTPOINTS_FIELD_NUMBER: builtins.int
    OUTPUTPATH_FIELD_NUMBER: builtins.int
    INCLUDECONTAINERLOGSONERROR_FIELD_NUMBER: builtins.int
    INCLUDECONTAINERLOGSONSUCCESS_FIELD_NUMBER: builtins.int
    MINIMUMCONTAINERMEMORYSIZE_FIELD_NUMBER: builtins.int
    EXTRACHUNKCACHESIZETOAVAILABLEMEMORYRATIO_FIELD_NUMBER: builtins.int
    PROXYCONFIGURATION_FIELD_NUMBER: builtins.int
    @property
    def command(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    @property
    def mountPoints(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___MountPoint]: ...
    outputPath: builtins.str
    includeContainerLogsOnError: builtins.bool
    includeContainerLogsOnSuccess: builtins.bool
    minimumContainerMemorySize: builtins.int
    """When executing a computation, the available VM memory is split into two:
    1. One part given to the in-memory chunk cache (this is backing the /input and /output filesystems, analogous to
        the kernel's pagecache).
    2. The second part is given to the container itself.
    The sizes are controlled by minimumContainerMemorySize and extraChunkCacheSizeToAvailableMemoryRatio.
    First minimumContainerMemorySize and a hardcoded minimum chunk cache size is subtracted from the available memory,
    then the rest is split according to extraChunkCacheSizeToAvailableMemoryRatio.
    For example, given a 64G VM with 62G available memory for compute:
    * minimumContainerMemorySize by default is 2G
    * minimum chunk cache size is 256M
    * extraChunkCacheSizeToAvailableMemoryRatio by default is 0.0625
    * therefore 0.0625 * (62G - 2G - 256M) =~ 3730M further memory is given to the chunk cache
    * so we end up with chunk_cache_size ~= 4G, container_memory ~=58G
    Generally speaking the split should be determined by the computation itself:
    * Example SQLite: SQLite is memory-bound generally speaking and does a lot of back-and-forth between its
        in-memory cache and the db file. This means that high extraChunkCacheSizeToAvailableMemoryRatio(1.0) and low
        minimumContainerMemorySize should be used because this will speed up the file backing, and SQLite doesn't use
        the extra container memory efficiently.
    * Example CHUV pipeline: this computation accesses sparse static input genome data in a fairly random manner,
        meaning that the best course of action is to read all data into memory first instead of relying on the chunk
        cache backed filesystem. This means low extraChunkCacheSizeToAvailableMemoryRatio(0.0) should be used.
        A setting of 0.0 means that all available extra memory (aside from the minimum chunk cache size) will be
        given to the container.
    * Example default settings: by default most but not all of the memory is given to the container, assuming that
        most applications tend to read the input files into memory as a first step instead of streaming through.
    default 2G
    """
    extraChunkCacheSizeToAvailableMemoryRatio: builtins.float
    """default 0.0625"""
    @property
    def proxyConfiguration(self) -> global___ProxyConfiguration:
        """Configure internet connectivity for the container. The VM starts a proxy (see delta-vm-proxy) which sets up
        networking that appears as if the container had direct internet access. However, all traffic is routed through
        this proxy, including DNS which returns fake IP addresses. The proxy is responsible for enforcing restrictive
        measures on how the internet can be accessed.
        null = disabled
        liberal = all traffic is allowed
        domain allowlist = only domains listed here can be accessed
        """
    def __init__(
        self,
        *,
        command: collections.abc.Iterable[builtins.str] | None = ...,
        mountPoints: collections.abc.Iterable[global___MountPoint] | None = ...,
        outputPath: builtins.str = ...,
        includeContainerLogsOnError: builtins.bool = ...,
        includeContainerLogsOnSuccess: builtins.bool = ...,
        minimumContainerMemorySize: builtins.int | None = ...,
        extraChunkCacheSizeToAvailableMemoryRatio: builtins.float | None = ...,
        proxyConfiguration: global___ProxyConfiguration | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["_extraChunkCacheSizeToAvailableMemoryRatio", b"_extraChunkCacheSizeToAvailableMemoryRatio", "_minimumContainerMemorySize", b"_minimumContainerMemorySize", "_proxyConfiguration", b"_proxyConfiguration", "extraChunkCacheSizeToAvailableMemoryRatio", b"extraChunkCacheSizeToAvailableMemoryRatio", "minimumContainerMemorySize", b"minimumContainerMemorySize", "proxyConfiguration", b"proxyConfiguration"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["_extraChunkCacheSizeToAvailableMemoryRatio", b"_extraChunkCacheSizeToAvailableMemoryRatio", "_minimumContainerMemorySize", b"_minimumContainerMemorySize", "_proxyConfiguration", b"_proxyConfiguration", "command", b"command", "extraChunkCacheSizeToAvailableMemoryRatio", b"extraChunkCacheSizeToAvailableMemoryRatio", "includeContainerLogsOnError", b"includeContainerLogsOnError", "includeContainerLogsOnSuccess", b"includeContainerLogsOnSuccess", "minimumContainerMemorySize", b"minimumContainerMemorySize", "mountPoints", b"mountPoints", "outputPath", b"outputPath", "proxyConfiguration", b"proxyConfiguration"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_extraChunkCacheSizeToAvailableMemoryRatio", b"_extraChunkCacheSizeToAvailableMemoryRatio"]) -> typing_extensions.Literal["extraChunkCacheSizeToAvailableMemoryRatio"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_minimumContainerMemorySize", b"_minimumContainerMemorySize"]) -> typing_extensions.Literal["minimumContainerMemorySize"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing_extensions.Literal["_proxyConfiguration", b"_proxyConfiguration"]) -> typing_extensions.Literal["proxyConfiguration"] | None: ...

global___StaticImage = StaticImage

@typing_extensions.final
class ProxyConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LIBERAL_FIELD_NUMBER: builtins.int
    DOMAINALLOWLIST_FIELD_NUMBER: builtins.int
    @property
    def liberal(self) -> global___ProxyConfigurationLiberal: ...
    @property
    def domainAllowlist(self) -> global___ProxyConfigurationDomainAllowlist: ...
    def __init__(
        self,
        *,
        liberal: global___ProxyConfigurationLiberal | None = ...,
        domainAllowlist: global___ProxyConfigurationDomainAllowlist | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["configuration", b"configuration", "domainAllowlist", b"domainAllowlist", "liberal", b"liberal"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["configuration", b"configuration", "domainAllowlist", b"domainAllowlist", "liberal", b"liberal"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["configuration", b"configuration"]) -> typing_extensions.Literal["liberal", "domainAllowlist"] | None: ...

global___ProxyConfiguration = ProxyConfiguration

@typing_extensions.final
class ProxyConfigurationLiberal(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___ProxyConfigurationLiberal = ProxyConfigurationLiberal

@typing_extensions.final
class ProxyConfigurationDomainAllowlist(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DOMAIN_FIELD_NUMBER: builtins.int
    @property
    def domain(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.str]: ...
    def __init__(
        self,
        *,
        domain: collections.abc.Iterable[builtins.str] | None = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["domain", b"domain"]) -> None: ...

global___ProxyConfigurationDomainAllowlist = ProxyConfigurationDomainAllowlist

@typing_extensions.final
class MountPoint(google.protobuf.message.Message):
    """Dependencies are mounted under the `/input` directory
    For example for a mount point entry { path: "/data", dependency: "dep" }
    the worker will mount the dependency `dep` at path `/input/data`
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PATH_FIELD_NUMBER: builtins.int
    DEPENDENCY_FIELD_NUMBER: builtins.int
    path: builtins.str
    dependency: builtins.str
    def __init__(
        self,
        *,
        path: builtins.str = ...,
        dependency: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["dependency", b"dependency", "path", b"path"]) -> None: ...

global___MountPoint = MountPoint



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: dataset_sink.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x12\x64\x61taset_sink.proto\x12\x0c\x64\x61taset_sink\"\t\n\x07RawFile\"\n\n\x08\x41llFiles\">\n\nSingleFile\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x15\n\x08renameAs\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\x0b\n\t_renameAs\"8\n\rFileSelection\x12\'\n\x05names\x18\x01 \x03(\x0b\x32\x18.dataset_sink.SingleFile\"k\n\x07ZipFile\x12%\n\x03\x61ll\x18\x01 \x01(\x0b\x32\x16.dataset_sink.AllFilesH\x00\x12,\n\x05\x66iles\x18\x02 \x01(\x0b\x32\x1b.dataset_sink.FileSelectionH\x00\x42\x0b\n\tselection\"\x81\x01\n\tSinkInput\x12\x12\n\ndependency\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12$\n\x03raw\x18\x03 \x01(\x0b\x32\x15.dataset_sink.RawFileH\x00\x12$\n\x03zip\x18\x04 \x01(\x0b\x32\x15.dataset_sink.ZipFileH\x00\x42\x06\n\x04\x66ile\"\xb5\x01\n\x1e\x44\x61tasetSinkWorkerConfiguration\x12\'\n\x06inputs\x18\x01 \x03(\x0b\x32\x17.dataset_sink.SinkInput\x12\x1f\n\x17\x65ncryptionKeyDependency\x18\x02 \x01(\t\x12\x1c\n\x0f\x64\x61tasetImportId\x18\x03 \x01(\tH\x00\x88\x01\x01\x12\x17\n\x0fisKeyHexEncoded\x18\x04 \x01(\x08\x42\x12\n\x10_datasetImportIdb\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'dataset_sink_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _RAWFILE._serialized_start=36
  _RAWFILE._serialized_end=45
  _ALLFILES._serialized_start=47
  _ALLFILES._serialized_end=57
  _SINGLEFILE._serialized_start=59
  _SINGLEFILE._serialized_end=121
  _FILESELECTION._serialized_start=123
  _FILESELECTION._serialized_end=179
  _ZIPFILE._serialized_start=181
  _ZIPFILE._serialized_end=288
  _SINKINPUT._serialized_start=291
  _SINKINPUT._serialized_end=420
  _DATASETSINKWORKERCONFIGURATION._serialized_start=423
  _DATASETSINKWORKERCONFIGURATION._serialized_end=604
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: identity_endorsement.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1aidentity_endorsement.proto\x12\x14identity_endorsement\"\xd2\x01\n\x12\x45ndorsementRequest\x12L\n\x15pkiEndorsementRequest\x18\x02 \x01(\x0b\x32+.identity_endorsement.PkiEndorsementRequestH\x00\x12X\n\x1b\x64\x63rSecretEndorsementRequest\x18\x03 \x01(\x0b\x32\x31.identity_endorsement.DcrSecretEndorsementRequestH\x00\x42\x14\n\x12\x65ndorsementRequest\"\xd8\x01\n\x13\x45ndorsementResponse\x12N\n\x16pkiEndorsementResponse\x18\x02 \x01(\x0b\x32,.identity_endorsement.PkiEndorsementResponseH\x00\x12Z\n\x1c\x64\x63rSecretEndorsementResponse\x18\x03 \x01(\x0b\x32\x32.identity_endorsement.DcrSecretEndorsementResponseH\x00\x42\x15\n\x13\x65ndorsementResponse\"4\n\x15PkiEndorsementRequest\x12\x1b\n\x13\x63\x65rtificateChainPem\x18\x01 \x01(\x0c\"Z\n\x16PkiEndorsementResponse\x12@\n\x0epkiEndorsement\x18\x01 \x01(\x0b\x32(.identity_endorsement.EnclaveEndorsement\"0\n\x1b\x44\x63rSecretEndorsementRequest\x12\x11\n\tdcrSecret\x18\x01 \x01(\t\"{\n\x1c\x44\x63rSecretEndorsementResponse\x12\x46\n\x14\x64\x63rSecretEndorsement\x18\x01 \x01(\x0b\x32(.identity_endorsement.EnclaveEndorsement\x12\x13\n\x0b\x64\x63rSecretId\x18\x02 \x01(\x0c\"\xca\x01\n\x13\x45nclaveEndorsements\x12=\n\x0bpersonalPki\x18\x01 \x01(\x0b\x32(.identity_endorsement.EnclaveEndorsement\x12\x37\n\x05\x64qPki\x18\x02 \x01(\x0b\x32(.identity_endorsement.EnclaveEndorsement\x12;\n\tdcrSecret\x18\x03 \x01(\x0b\x32(.identity_endorsement.EnclaveEndorsement\"7\n\x12\x45nclaveEndorsement\x12!\n\x19\x65ndorsementCertificateDer\x18\x01 \x01(\x0c\"&\n\x08PkiClaim\x12\x1a\n\x12rootCertificateDer\x18\x01 \x01(\x0c\"%\n\x0e\x44\x63rSecretClaim\x12\x13\n\x0b\x64\x63rSecretId\x18\x01 \x01(\x0c\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'identity_endorsement_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _ENDORSEMENTREQUEST._serialized_start=53
  _ENDORSEMENTREQUEST._serialized_end=263
  _ENDORSEMENTRESPONSE._serialized_start=266
  _ENDORSEMENTRESPONSE._serialized_end=482
  _PKIENDORSEMENTREQUEST._serialized_start=484
  _PKIENDORSEMENTREQUEST._serialized_end=536
  _PKIENDORSEMENTRESPONSE._serialized_start=538
  _PKIENDORSEMENTRESPONSE._serialized_end=628
  _DCRSECRETENDORSEMENTREQUEST._serialized_start=630
  _DCRSECRETENDORSEMENTREQUEST._serialized_end=678
  _DCRSECRETENDORSEMENTRESPONSE._serialized_start=680
  _DCRSECRETENDORSEMENTRESPONSE._serialized_end=803
  _ENCLAVEENDORSEMENTS._serialized_start=806
  _ENCLAVEENDORSEMENTS._serialized_end=1008
  _ENCLAVEENDORSEMENT._serialized_start=1010
  _ENCLAVEENDORSEMENT._serialized_end=1065
  _PKICLAIM._serialized_start=1067
  _PKICLAIM._serialized_end=1105
  _DCRSECRETCLAIM._serialized_start=1107
  _DCRSECRETCLAIM._serialized_end=1144
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: data_room.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import builder as _builder
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from . import attestation_pb2 as attestation__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0f\x64\x61ta_room.proto\x12\tdata_room\x1a\x11\x61ttestation.proto\"\xb4\x01\n\x08\x44\x61taRoom\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x03 \x01(\t\x12\x39\n\x12governanceProtocol\x18\x04 \x01(\x0b\x32\x1d.data_room.GovernanceProtocol\x12>\n\x14initialConfiguration\x18\x05 \x01(\x0b\x32 .data_room.DataRoomConfiguration\"\xb6\x01\n\x12GovernanceProtocol\x12?\n\x14staticDataRoomPolicy\x18\x01 \x01(\x0b\x32\x1f.data_room.StaticDataRoomPolicyH\x00\x12U\n\x1f\x61\x66\x66\x65\x63tedDataOwnersApprovePolicy\x18\x02 \x01(\x0b\x32*.data_room.AffectedDataOwnersApprovePolicyH\x00\x42\x08\n\x06policy\"\x16\n\x14StaticDataRoomPolicy\"!\n\x1f\x41\x66\x66\x65\x63tedDataOwnersApprovePolicy\"J\n\x15\x44\x61taRoomConfiguration\x12\x31\n\x08\x65lements\x18\x01 \x03(\x0b\x32\x1f.data_room.ConfigurationElement\"\x9d\x02\n\x14\x43onfigurationElement\x12\n\n\x02id\x18\x01 \x01(\t\x12-\n\x0b\x63omputeNode\x18\x02 \x01(\x0b\x32\x16.data_room.ComputeNodeH\x00\x12I\n\x18\x61ttestationSpecification\x18\x03 \x01(\x0b\x32%.attestation.AttestationSpecificationH\x00\x12\x33\n\x0euserPermission\x18\x04 \x01(\x0b\x32\x19.data_room.UserPermissionH\x00\x12?\n\x14\x61uthenticationMethod\x18\x05 \x01(\x0b\x32\x1f.data_room.AuthenticationMethodH\x00\x42\t\n\x07\x65lement\"\xb8\x01\n\x19\x43onfigurationModification\x12)\n\x03\x61\x64\x64\x18\x01 \x01(\x0b\x32\x1a.data_room.AddModificationH\x00\x12/\n\x06\x63hange\x18\x02 \x01(\x0b\x32\x1d.data_room.ChangeModificationH\x00\x12/\n\x06\x64\x65lete\x18\x03 \x01(\x0b\x32\x1d.data_room.DeleteModificationH\x00\x42\x0e\n\x0cmodification\"C\n\x0f\x41\x64\x64Modification\x12\x30\n\x07\x65lement\x18\x01 \x01(\x0b\x32\x1f.data_room.ConfigurationElement\"F\n\x12\x43hangeModification\x12\x30\n\x07\x65lement\x18\x01 \x01(\x0b\x32\x1f.data_room.ConfigurationElement\" \n\x12\x44\x65leteModification\x12\n\n\x02id\x18\x01 \x01(\t\"\x9c\x01\n\x13\x43onfigurationCommit\x12\n\n\x02id\x18\x01 \x01(\t\x12\x0c\n\x04name\x18\x02 \x01(\t\x12\x12\n\ndataRoomId\x18\x03 \x01(\x0c\x12\x1a\n\x12\x64\x61taRoomHistoryPin\x18\x04 \x01(\x0c\x12;\n\rmodifications\x18\x05 \x03(\x0b\x32$.data_room.ConfigurationModification\"O\n\x18WindowRateLimitingConfig\x12\x19\n\x11timeWindowSeconds\x18\x01 \x01(\r\x12\x18\n\x10numMaxExecutions\x18\x02 \x01(\r\"U\n\x12RateLimitingConfig\x12\x35\n\x06window\x18\x01 \x01(\x0b\x32#.data_room.WindowRateLimitingConfigH\x00\x42\x08\n\x06method\"\xb6\x02\n\x0b\x43omputeNode\x12\x10\n\x08nodeName\x18\x01 \x01(\t\x12*\n\x04leaf\x18\x02 \x01(\x0b\x32\x1a.data_room.ComputeNodeLeafH\x00\x12\x34\n\tparameter\x18\x04 \x01(\x0b\x32\x1f.data_room.ComputeNodeParameterH\x00\x12.\n\x06\x62ranch\x18\x03 \x01(\x0b\x32\x1c.data_room.ComputeNodeBranchH\x00\x12\x30\n\x07\x61irlock\x18\x06 \x01(\x0b\x32\x1d.data_room.ComputeNodeAirlockH\x00\x12\x38\n\x0crateLimiting\x18\x05 \x01(\x0b\x32\x1d.data_room.RateLimitingConfigH\x01\x88\x01\x01\x42\x06\n\x04nodeB\x0f\n\r_rateLimiting\"%\n\x0f\x43omputeNodeLeaf\x12\x12\n\nisRequired\x18\x01 \x01(\x08\"*\n\x14\x43omputeNodeParameter\x12\x12\n\nisRequired\x18\x01 \x01(\x08\"E\n\x12\x43omputeNodeAirlock\x12\x12\n\nquotaBytes\x18\x01 \x01(\x04\x12\x1b\n\x13\x61irlockedDependency\x18\x02 \x01(\t\"&\n\x13\x43omputeNodeProtocol\x12\x0f\n\x07version\x18\x01 \x01(\r\"\xc3\x01\n\x11\x43omputeNodeBranch\x12\x0e\n\x06\x63onfig\x18\x01 \x01(\x0c\x12\x14\n\x0c\x64\x65pendencies\x18\x02 \x03(\t\x12\x32\n\x0coutputFormat\x18\x03 \x01(\x0e\x32\x1c.data_room.ComputeNodeFormat\x12\x30\n\x08protocol\x18\x04 \x01(\x0b\x32\x1e.data_room.ComputeNodeProtocol\x12\"\n\x1a\x61ttestationSpecificationId\x18\x05 \x01(\t\"k\n\x0eUserPermission\x12\r\n\x05\x65mail\x18\x01 \x01(\t\x12*\n\x0bpermissions\x18\x02 \x03(\x0b\x32\x15.data_room.Permission\x12\x1e\n\x16\x61uthenticationMethodId\x18\x03 \x01(\t\"\x95\x01\n\x14\x41uthenticationMethod\x12)\n\x0bpersonalPki\x18\x01 \x01(\x0b\x32\x14.data_room.PkiPolicy\x12#\n\x05\x64qPki\x18\x02 \x01(\x0b\x32\x14.data_room.PkiPolicy\x12-\n\tdcrSecret\x18\x03 \x01(\x0b\x32\x1a.data_room.DcrSecretPolicy\"\'\n\tPkiPolicy\x12\x1a\n\x12rootCertificatePem\x18\x01 \x01(\x0c\"&\n\x0f\x44\x63rSecretPolicy\x12\x13\n\x0b\x64\x63rSecretId\x18\x01 \x01(\x0c\"\x8c\t\n\nPermission\x12G\n\x18\x65xecuteComputePermission\x18\x01 \x01(\x0b\x32#.data_room.ExecuteComputePermissionH\x00\x12;\n\x12leafCrudPermission\x18\x02 \x01(\x0b\x32\x1d.data_room.LeafCrudPermissionH\x00\x12K\n\x1aretrieveDataRoomPermission\x18\x03 \x01(\x0b\x32%.data_room.RetrieveDataRoomPermissionH\x00\x12K\n\x1aretrieveAuditLogPermission\x18\x04 \x01(\x0b\x32%.data_room.RetrieveAuditLogPermissionH\x00\x12W\n retrieveDataRoomStatusPermission\x18\x05 \x01(\x0b\x32+.data_room.RetrieveDataRoomStatusPermissionH\x00\x12S\n\x1eupdateDataRoomStatusPermission\x18\x06 \x01(\x0b\x32).data_room.UpdateDataRoomStatusPermissionH\x00\x12]\n#retrievePublishedDatasetsPermission\x18\x07 \x01(\x0b\x32..data_room.RetrievePublishedDatasetsPermissionH\x00\x12\x37\n\x10\x64ryRunPermission\x18\x08 \x01(\x0b\x32\x1b.data_room.DryRunPermissionH\x00\x12W\n generateMergeSignaturePermission\x18\t \x01(\x0b\x32+.data_room.GenerateMergeSignaturePermissionH\x00\x12]\n#executeDevelopmentComputePermission\x18\n \x01(\x0b\x32..data_room.ExecuteDevelopmentComputePermissionH\x00\x12[\n\"mergeConfigurationCommitPermission\x18\x0b \x01(\x0b\x32-.data_room.MergeConfigurationCommitPermissionH\x00\x12U\n\x1fretrieveComputeResultPermission\x18\x0c \x01(\x0b\x32*.data_room.RetrieveComputeResultPermissionH\x00\x12M\n\x1b\x63\x61sAuxiliaryStatePermission\x18\r \x01(\x0b\x32&.data_room.CasAuxiliaryStatePermissionH\x00\x12O\n\x1creadAuxiliaryStatePermission\x18\x0e \x01(\x0b\x32\'.data_room.ReadAuxiliaryStatePermissionH\x00\x42\x0c\n\npermission\"1\n\x18\x45xecuteComputePermission\x12\x15\n\rcomputeNodeId\x18\x01 \x01(\t\"(\n\x12LeafCrudPermission\x12\x12\n\nleafNodeId\x18\x01 \x01(\t\"\x1c\n\x1aRetrieveDataRoomPermission\"\x1c\n\x1aRetrieveAuditLogPermission\"\"\n RetrieveDataRoomStatusPermission\" \n\x1eUpdateDataRoomStatusPermission\"%\n#RetrievePublishedDatasetsPermission\"\x12\n\x10\x44ryRunPermission\"\"\n GenerateMergeSignaturePermission\"%\n#ExecuteDevelopmentComputePermission\"$\n\"MergeConfigurationCommitPermission\"8\n\x1fRetrieveComputeResultPermission\x12\x15\n\rcomputeNodeId\x18\x01 \x01(\t\"\x1d\n\x1b\x43\x61sAuxiliaryStatePermission\"\x1e\n\x1cReadAuxiliaryStatePermission*%\n\x11\x43omputeNodeFormat\x12\x07\n\x03RAW\x10\x00\x12\x07\n\x03ZIP\x10\x01\x62\x06proto3')

_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, globals())
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'data_room_pb2', globals())
if _descriptor._USE_C_DESCRIPTORS == False:

  DESCRIPTOR._options = None
  _COMPUTENODEFORMAT._serialized_start=4259
  _COMPUTENODEFORMAT._serialized_end=4296
  _DATAROOM._serialized_start=50
  _DATAROOM._serialized_end=230
  _GOVERNANCEPROTOCOL._serialized_start=233
  _GOVERNANCEPROTOCOL._serialized_end=415
  _STATICDATAROOMPOLICY._serialized_start=417
  _STATICDATAROOMPOLICY._serialized_end=439
  _AFFECTEDDATAOWNERSAPPROVEPOLICY._serialized_start=441
  _AFFECTEDDATAOWNERSAPPROVEPOLICY._serialized_end=474
  _DATAROOMCONFIGURATION._serialized_start=476
  _DATAROOMCONFIGURATION._serialized_end=550
  _CONFIGURATIONELEMENT._serialized_start=553
  _CONFIGURATIONELEMENT._serialized_end=838
  _CONFIGURATIONMODIFICATION._serialized_start=841
  _CONFIGURATIONMODIFICATION._serialized_end=1025
  _ADDMODIFICATION._serialized_start=1027
  _ADDMODIFICATION._serialized_end=1094
  _CHANGEMODIFICATION._serialized_start=1096
  _CHANGEMODIFICATION._serialized_end=1166
  _DELETEMODIFICATION._serialized_start=1168
  _DELETEMODIFICATION._serialized_end=1200
  _CONFIGURATIONCOMMIT._serialized_start=1203
  _CONFIGURATIONCOMMIT._serialized_end=1359
  _WINDOWRATELIMITINGCONFIG._serialized_start=1361
  _WINDOWRATELIMITINGCONFIG._serialized_end=1440
  _RATELIMITINGCONFIG._serialized_start=1442
  _RATELIMITINGCONFIG._serialized_end=1527
  _COMPUTENODE._serialized_start=1530
  _COMPUTENODE._serialized_end=1840
  _COMPUTENODELEAF._serialized_start=1842
  _COMPUTENODELEAF._serialized_end=1879
  _COMPUTENODEPARAMETER._serialized_start=1881
  _COMPUTENODEPARAMETER._serialized_end=1923
  _COMPUTENODEAIRLOCK._serialized_start=1925
  _COMPUTENODEAIRLOCK._serialized_end=1994
  _COMPUTENODEPROTOCOL._serialized_start=1996
  _COMPUTENODEPROTOCOL._serialized_end=2034
  _COMPUTENODEBRANCH._serialized_start=2037
  _COMPUTENODEBRANCH._serialized_end=2232
  _USERPERMISSION._serialized_start=2234
  _USERPERMISSION._serialized_end=2341
  _AUTHENTICATIONMETHOD._serialized_start=2344
  _AUTHENTICATIONMETHOD._serialized_end=2493
  _PKIPOLICY._serialized_start=2495
  _PKIPOLICY._serialized_end=2534
  _DCRSECRETPOLICY._serialized_start=2536
  _DCRSECRETPOLICY._serialized_end=2574
  _PERMISSION._serialized_start=2577
  _PERMISSION._serialized_end=3741
  _EXECUTECOMPUTEPERMISSION._serialized_start=3743
  _EXECUTECOMPUTEPERMISSION._serialized_end=3792
  _LEAFCRUDPERMISSION._serialized_start=3794
  _LEAFCRUDPERMISSION._serialized_end=3834
  _RETRIEVEDATAROOMPERMISSION._serialized_start=3836
  _RETRIEVEDATAROOMPERMISSION._serialized_end=3864
  _RETRIEVEAUDITLOGPERMISSION._serialized_start=3866
  _RETRIEVEAUDITLOGPERMISSION._serialized_end=3894
  _RETRIEVEDATAROOMSTATUSPERMISSION._serialized_start=3896
  _RETRIEVEDATAROOMSTATUSPERMISSION._serialized_end=3930
  _UPDATEDATAROOMSTATUSPERMISSION._serialized_start=3932
  _UPDATEDATAROOMSTATUSPERMISSION._serialized_end=3964
  _RETRIEVEPUBLISHEDDATASETSPERMISSION._serialized_start=3966
  _RETRIEVEPUBLISHEDDATASETSPERMISSION._serialized_end=4003
  _DRYRUNPERMISSION._serialized_start=4005
  _DRYRUNPERMISSION._serialized_end=4023
  _GENERATEMERGESIGNATUREPERMISSION._serialized_start=4025
  _GENERATEMERGESIGNATUREPERMISSION._serialized_end=4059
  _EXECUTEDEVELOPMENTCOMPUTEPERMISSION._serialized_start=4061
  _EXECUTEDEVELOPMENTCOMPUTEPERMISSION._serialized_end=4098
  _MERGECONFIGURATIONCOMMITPERMISSION._serialized_start=4100
  _MERGECONFIGURATIONCOMMITPERMISSION._serialized_end=4136
  _RETRIEVECOMPUTERESULTPERMISSION._serialized_start=4138
  _RETRIEVECOMPUTERESULTPERMISSION._serialized_end=4194
  _CASAUXILIARYSTATEPERMISSION._serialized_start=4196
  _CASAUXILIARYSTATEPERMISSION._serialized_end=4225
  _READAUXILIARYSTATEPERMISSION._serialized_start=4227
  _READAUXILIARYSTATEPERMISSION._serialized_end=4257
# @@protoc_insertion_point(module_scope)



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""
import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _S3Provider:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _S3ProviderEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_S3Provider.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    AWS: _S3Provider.ValueType  # 0
    GCS: _S3Provider.ValueType  # 1

class S3Provider(_S3Provider, metaclass=_S3ProviderEnumTypeWrapper): ...

AWS: S3Provider.ValueType  # 0
GCS: S3Provider.ValueType  # 1
global___S3Provider = S3Provider

@typing_extensions.final
class S3SinkWorkerConfiguration(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ENDPOINT_FIELD_NUMBER: builtins.int
    REGION_FIELD_NUMBER: builtins.int
    CREDENTIALSDEPENDENCY_FIELD_NUMBER: builtins.int
    OBJECTS_FIELD_NUMBER: builtins.int
    S3PROVIDER_FIELD_NUMBER: builtins.int
    endpoint: builtins.str
    region: builtins.str
    """S3 region can be left empty for a GCS sink worker"""
    credentialsDependency: builtins.str
    @property
    def objects(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[global___S3Object]: ...
    s3Provider: global___S3Provider.ValueType
    def __init__(
        self,
        *,
        endpoint: builtins.str = ...,
        region: builtins.str = ...,
        credentialsDependency: builtins.str = ...,
        objects: collections.abc.Iterable[global___S3Object] | None = ...,
        s3Provider: global___S3Provider.ValueType = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["credentialsDependency", b"credentialsDependency", "endpoint", b"endpoint", "objects", b"objects", "region", b"region", "s3Provider", b"s3Provider"]) -> None: ...

global___S3SinkWorkerConfiguration = S3SinkWorkerConfiguration

@typing_extensions.final
class S3Object(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    DEPENDENCY_FIELD_NUMBER: builtins.int
    ZIP_FIELD_NUMBER: builtins.int
    RAW_FIELD_NUMBER: builtins.int
    dependency: builtins.str
    @property
    def zip(self) -> global___ZipObject: ...
    @property
    def raw(self) -> global___RawObject: ...
    def __init__(
        self,
        *,
        dependency: builtins.str = ...,
        zip: global___ZipObject | None = ...,
        raw: global___RawObject | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["format", b"format", "raw", b"raw", "zip", b"zip"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["dependency", b"dependency", "format", b"format", "raw", b"raw", "zip", b"zip"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["format", b"format"]) -> typing_extensions.Literal["zip", "raw"] | None: ...

global___S3Object = S3Object

@typing_extensions.final
class RawObject(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEY_FIELD_NUMBER: builtins.int
    key: builtins.str
    def __init__(
        self,
        *,
        key: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["key", b"key"]) -> None: ...

global___RawObject = RawObject

@typing_extensions.final
class ZipObject(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SINGLEFILE_FIELD_NUMBER: builtins.int
    FULLCONTENT_FIELD_NUMBER: builtins.int
    @property
    def singleFile(self) -> global___SingleFile: ...
    @property
    def fullContent(self) -> global___FullContent: ...
    def __init__(
        self,
        *,
        singleFile: global___SingleFile | None = ...,
        fullContent: global___FullContent | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing_extensions.Literal["fullContent", b"fullContent", "kind", b"kind", "singleFile", b"singleFile"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing_extensions.Literal["fullContent", b"fullContent", "kind", b"kind", "singleFile", b"singleFile"]) -> None: ...
    def WhichOneof(self, oneof_group: typing_extensions.Literal["kind", b"kind"]) -> typing_extensions.Literal["singleFile", "fullContent"] | None: ...

global___ZipObject = ZipObject

@typing_extensions.final
class SingleFile(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    KEY_FIELD_NUMBER: builtins.int
    PATH_FIELD_NUMBER: builtins.int
    key: builtins.str
    path: builtins.str
    def __init__(
        self,
        *,
        key: builtins.str = ...,
        path: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["key", b"key", "path", b"path"]) -> None: ...

global___SingleFile = SingleFile

@typing_extensions.final
class FullContent(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    def __init__(
        self,
    ) -> None: ...

global___FullContent = FullContent

@typing_extensions.final
class S3Credentials(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ACCESSKEY_FIELD_NUMBER: builtins.int
    SECRETKEY_FIELD_NUMBER: builtins.int
    accessKey: builtins.str
    secretKey: builtins.str
    def __init__(
        self,
        *,
        accessKey: builtins.str = ...,
        secretKey: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing_extensions.Literal["accessKey", b"accessKey", "secretKey", b"secretKey"]) -> None: ...

global___S3Credentials = S3Credentials



import base64
import io
import json
from typing import Dict, Mapping, Optional, Text
from uuid import uuid4
import uuid
import zipfile
from ..client import Client
from ..types import (
    DataLabDatasetType,
    DataLabDefinition,
    DryRunOptions,
    EnclaveSpecification,
    JobId,
    MatchingId,
    MatchingIdFormat,
    TableColumnHashingAlgorithm,
)
from decentriq_dcr_compiler import compiler
from decentriq_dcr_compiler.schemas.lookalike_media_data_room import (
    LookalikeMediaDataRoom,
)
from decentriq_dcr_compiler.schemas.create_data_lab import (
    CreateDataLab,
    CreateDataLab,
    CreateDataLabItem1,
    CreateDataLabComputeV1,
)
from ..proto import DataRoom, CreateDcrPurpose
from ..proto.length_delimited import parse_length_delimited, serialize_length_delimited
from ..storage import Key
from ..keychain import Keychain, KeychainEntry
from ..session import LATEST_GCG_PROTOCOL_VERSION, Session
from ..helpers import (
    get_latest_enclave_specs_as_dictionary,
    create_session_from_driver_spec,
)
from ..lookup_tables import MATCHING_ID_INTERNAL_LOOKUP
from pathlib import Path

__all__ = [
    "MatchingId",
    "provision_local_datasets",
    "run",
    "get_validation_report",
    "get_statistics_report",
    "provision_to_lookalike_media_data_room",
]


class Dataset:
    def __init__(self, manifest_hash: str, key: Key):
        self.manifest_hash = manifest_hash
        self.key = key


class DataLabConfig:
    def __init__(
        self,
        name: str,
        has_demographics: bool,
        has_embeddings: bool,
        num_embeddings: int,
        matching_id: MatchingId,
    ):
        self.name = name
        self.has_demographics = has_demographics
        self.has_embeddings = has_embeddings
        self.num_embeddings = num_embeddings
        self.matching_id = matching_id


class ExistingDataLab:
    def __init__(
        self,
        data_lab_definition: DataLabDefinition,
        keychain: Keychain,
    ):
        self.keychain = keychain
        self.id = data_lab_definition["id"]
        self.high_level_representation = data_lab_definition[
            "highLevelRepresentationAsString"
        ]
        self.match_dataset = data_lab_definition["usersDataset"]
        self.segments_dataset = data_lab_definition["segmentsDataset"]
        self.demographics_dataset = data_lab_definition["demographicsDataset"]
        self.embeddings_dataset = data_lab_definition["embeddingsDataset"]


class DataLab:
    def __init__(
        self,
        client: Client,
        cfg: DataLabConfig,
        existing_data_lab: ExistingDataLab = None,
    ):
        self.client = client
        enclave_specs = get_latest_enclave_specs_as_dictionary(self.client)
        auth, _ = client.create_auth_using_decentriq_pki(enclave_specs)
        self.session = client.create_session(auth, enclave_specs)
        self.cfg = cfg
        self.dcr_id = ""
        self.datasets: Dict[DataLabDatasetType, Dataset] = dict()

        if existing_data_lab:
            # Create a DataLab using the existing configuration.
            self.data_lab_id = existing_data_lab.id
            self.hl_data_lab = compiler.DataLab.parse_obj(
                json.loads(existing_data_lab.high_level_representation)
            )
            self._populate_datasets_cache_from_existing(existing_data_lab)
        else:
            # Create a new DataLab.
            self.data_lab_id = f"DataLab-{str(uuid.uuid4())}"
            (
                matching_id_format,
                matching_id_hashing_algorithm,
            ) = MATCHING_ID_INTERNAL_LOOKUP[self.cfg.matching_id]
            create_data_lab = CreateDataLab(
                __root__=CreateDataLabItem1(
                    v1=CreateDataLabComputeV1(
                        authenticationRootCertificatePem=self.client.decentriq_ca_root_certificate.decode(),
                        driverEnclaveSpecification=EnclaveSpecification(
                            attestationProtoBase64="",
                            id="",
                            workerProtocol=0,
                        ),
                        hasDemographics=self.cfg.has_demographics,
                        hasEmbeddings=self.cfg.has_embeddings,
                        id=self.data_lab_id,
                        matchingIdFormat=matching_id_format,
                        matchingIdHashingAlgorithm=matching_id_hashing_algorithm,
                        name=self.cfg.name,
                        numEmbeddings=self.cfg.num_embeddings,
                        publisherEmail=self.client.user_email,
                        pythonEnclaveSpecification=EnclaveSpecification(
                            attestationProtoBase64="",
                            id="",
                            workerProtocol=0,
                        ),
                    ),
                )
            )
            self.hl_data_lab = compiler.create_data_lab(create_data_lab)
            self.data_lab_id = self.client._publish_data_lab_from_existing(
                json.loads(self.hl_data_lab.json())
            )

    def _populate_datasets_cache_from_existing(
        self, existing_data_lab: ExistingDataLab
    ):
        keychain = existing_data_lab.keychain
        match_dataset = existing_data_lab.match_dataset
        if match_dataset is not None:
            manifest_hash = match_dataset["manifestHash"]
            key = keychain.get("dataset_key", manifest_hash)
            self.datasets[DataLabDatasetType.MATCH] = Dataset(
                manifest_hash, Key(key.value)
            )

        segments_dataset = existing_data_lab.segments_dataset
        if segments_dataset is not None:
            manifest_hash = segments_dataset["manifestHash"]
            key = keychain.get("dataset_key", manifest_hash)
            self.datasets[DataLabDatasetType.SEGMENTS] = Dataset(
                manifest_hash, Key(key.value)
            )

        demographics_dataset = existing_data_lab.demographics_dataset
        if self.cfg.has_demographics and demographics_dataset is not None:
            manifest_hash = demographics_dataset["manifestHash"]
            key = keychain.get("dataset_key", manifest_hash)
            self.datasets[DataLabDatasetType.DEMOGRAPHICS] = Dataset(
                manifest_hash, Key(key.value)
            )

        embeddings_dataset = existing_data_lab.embeddings_dataset
        if self.cfg.has_embeddings and embeddings_dataset is not None:
            manifest_hash = embeddings_dataset["manifestHash"]
            key = keychain.get("dataset_key", manifest_hash)
            self.datasets[DataLabDatasetType.EMBEDDINGS] = Dataset(
                manifest_hash, Key(key.value)
            )

    def _get_data_lab_enclave_specs(
        self,
        enclave_specs: Dict[str, EnclaveSpecification],
    ) -> (EnclaveSpecification, EnclaveSpecification):
        driver_spec = {}
        python_spec = {}
        for spec_id, spec in enclave_specs.items():
            spec_payload = {
                "attestationProtoBase64": base64.b64encode(
                    serialize_length_delimited(spec["proto"])
                ).decode(),
                "id": spec_id,
                "workerProtocol": spec["workerProtocols"][0],
            }
            if "decentriq.driver" in spec_id:
                spec["clientProtocols"] = [LATEST_GCG_PROTOCOL_VERSION]
                driver_spec = compiler.EnclaveSpecification.parse_obj(spec_payload)
            elif "decentriq.python-ml-worker" in spec_id:
                spec["clientProtocols"] = [LATEST_GCG_PROTOCOL_VERSION]
                python_spec = compiler.EnclaveSpecification.parse_obj(spec_payload)
        return (driver_spec, python_spec)

    def provision_local_datasets(
        self,
        key: Key,
        keychain: Keychain,
        matching_data_path: str,
        segments_data_path: str,
        demographics_data_path: Optional[str] = None,
        embeddings_data_path: Optional[str] = None,
    ):
        """
        Upload local datasets to the keychain and provision to the DataLab.

        **Parameters**
        - `key`: The key used to encrypt the dataset.
        - `keychain`: The keychain where the key will be stored.
        - `match`: The file path to the "match" dataset.
        - `segments`: The file path to the "segments" dataset.
        - `demographics`: The file path to the "demographics" dataset.
        - `embeddings`: The file path to the "embeddings" dataset.
        """
        if matching_data_path is not None:
            dataset_name = Path(matching_data_path).stem
            dataset_id = self._upload_dataset_to_keychain(
                matching_data_path, dataset_name, key, keychain
            )
            self.provision_dataset(dataset_id, key, DataLabDatasetType.MATCH)
        if segments_data_path is not None:
            dataset_name = Path(segments_data_path).stem
            dataset_id = self._upload_dataset_to_keychain(
                segments_data_path, dataset_name, key, keychain
            )
            self.provision_dataset(dataset_id, key, DataLabDatasetType.SEGMENTS)
        if embeddings_data_path is not None:
            dataset_name = Path(embeddings_data_path).stem
            self._check_dataset_type_permitted(
                dataset_type=DataLabDatasetType.EMBEDDINGS
            )
            dataset_id = self._upload_dataset_to_keychain(
                embeddings_data_path, dataset_name, key, keychain
            )
            self.provision_dataset(dataset_id, key, DataLabDatasetType.EMBEDDINGS)
        if demographics_data_path is not None:
            dataset_name = Path(demographics_data_path).stem
            self._check_dataset_type_permitted(
                dataset_type=DataLabDatasetType.DEMOGRAPHICS
            )
            dataset_id = self._upload_dataset_to_keychain(
                demographics_data_path, dataset_name, key, keychain
            )
            self.provision_dataset(dataset_id, key, DataLabDatasetType.DEMOGRAPHICS)

    def _upload_dataset_to_keychain(
        self, file_path: str, name: str, key: Key, keychain: Keychain
    ):
        with open(file_path, "rb") as file:
            dataset_id = self.client.upload_dataset(file, key, name)
            keychain.insert(KeychainEntry("dataset_key", dataset_id, key.material))
            return dataset_id

    def provision_dataset(
        self, manifest_hash: str, key: Key, dataset_type: DataLabDatasetType
    ):
        self._check_dataset_type_permitted(dataset_type=dataset_type)

        if dataset_type == DataLabDatasetType.EMBEDDINGS:
            self.client._set_datalab_embeddings_dataset(self.data_lab_id, manifest_hash)
        elif dataset_type == DataLabDatasetType.DEMOGRAPHICS:
            self.client._set_datalab_demographics_dataset(
                self.data_lab_id, manifest_hash
            )
        elif dataset_type == DataLabDatasetType.MATCH:
            self.client._set_datalab_matching_dataset(self.data_lab_id, manifest_hash)
        elif dataset_type == DataLabDatasetType.SEGMENTS:
            self.client._set_datalab_segments_dataset(self.data_lab_id, manifest_hash)
        else:
            raise Exception(f"Unknown dataset type {dataset_type}")

        # Store the provisioned datasets in the local cache.
        self.datasets[dataset_type] = Dataset(manifest_hash, key)

    def deprovision_dataset(self, dataset_type: DataLabDatasetType):
        self._check_dataset_type_permitted(dataset_type=dataset_type)

        # Sending "None" as the manifest hash causes the dataset to be deprovisioned.
        if dataset_type == DataLabDatasetType.EMBEDDINGS:
            self.client._set_datalab_embeddings_dataset(self.data_lab_id, None)
        elif dataset_type == DataLabDatasetType.DEMOGRAPHICS:
            self.client._set_datalab_demographics_dataset(self.data_lab_id, None)
        elif dataset_type == DataLabDatasetType.MATCH:
            self.client._set_datalab_matching_dataset(self.data_lab_id, None)
        elif dataset_type == DataLabDatasetType.SEGMENTS:
            self.client._set_datalab_segments_dataset(self.data_lab_id, None)
        else:
            raise Exception(f"Unknown dataset type {dataset_type}")

    def _check_dataset_type_permitted(self, dataset_type: DataLabDatasetType):
        if dataset_type == DataLabDatasetType.EMBEDDINGS:
            if not self.cfg.has_embeddings:
                raise Exception("Embeddings not enabled")
        elif dataset_type == DataLabDatasetType.DEMOGRAPHICS:
            if not self.cfg.has_demographics:
                raise Exception("Demographics not enabled")

    def run(
        self,
        /,
        *,
        dry_run: Optional[DryRunOptions] = None,
        parameters: Optional[Mapping[Text, Text]] = None,
    ):
        """
        Running the DataLab results in the validation jobs and statistics job being kicked off. 
        This function does not block waiting for the results. Instead the user should call the 
        `get_validation_report` or `get_statistics_report` function.
        """
        features = self._get_features()
        if "COMPUTE_STATISTICS" not in features:
            raise Exception("DataLab does not support computing statistics")

        self._update_enclave_specs()
        self.dcr_id = self._construct_backing_dcr(self.session)
        for dataset_type, dataset in self.datasets.items():
            node_name = self._get_data_lab_node_names(dataset_type)
            self.session.publish_dataset(
                self.dcr_id, dataset.manifest_hash, node_name, dataset.key
            )
        # Start validation jobs
        validation_job_id = self.session._submit_compute(
            self.dcr_id,
            self._get_validation_nodes(),
            dry_run=dry_run,
            parameters=parameters,
        ).jobId.hex()
        # Start statistics job
        statistics_job_id = self.session._submit_compute(
            self.dcr_id, ["publisher_data_statistics"]
        ).jobId.hex()

        self.client._set_datalab_job_ids(
            self.data_lab_id,
            validation_job_id,
            statistics_job_id,
            self.session.driver_attestation_specification_hash,
        )

    def get_validation_report(self, timeout: int = None):
        """
        Retrieve the validation report. This function will block until the report is ready unless a timeout is specified.

        **Parameters**:
        - `timeout`: Amount of time to wait (in seconds) for the validation report to become available.
        """
        data_lab = self.client.get_data_lab(self.data_lab_id)

        # A validation report is not stored in the DB if statistics have already been produced.
        if data_lab["statistics"]:
            raise Exception(
                "Cannot retrieve validation report. Statistics already calculated."
            )

        driver_attestation_hash = data_lab["jobsDriverAttestationHash"]
        if not driver_attestation_hash:
            raise Exception("Driver attestation spec not found")

        validation_compute_job_id = data_lab["validationComputeJobId"]
        if not validation_compute_job_id:
            raise Exception(
                "Validation compute job ID not found. Please run the DataLab."
            )

        driver_spec = self.client._get_enclave_spec_from_hash(driver_attestation_hash)
        if not driver_spec:
            raise Exception(f"Failed to find driver for data lab {self.data_lab_id}")

        session = create_session_from_driver_spec(self.client, driver_spec)
        validation_nodes = self._get_validation_nodes()
        session.wait_until_computation_has_finished_for_all_compute_nodes(
            validation_compute_job_id, validation_nodes, timeout=timeout
        )
        # Get individual reports
        validation_reports = {}
        for node in validation_nodes:
            job_id = JobId(validation_compute_job_id, node)
            result = session.get_computation_result(job_id, timeout=timeout)
            zip = zipfile.ZipFile(io.BytesIO(result), "r")
            if "validation-report.json" in zip.namelist():
                validation_reports[
                    node.removesuffix("_validation_report")
                ] = json.loads(zip.read("validation-report.json").decode())
        return validation_reports

    def get_statistics_report(self, timeout: int = None):
        """
        Retrieve the statistics report. This function will block until the report is ready unless a timeout is specified.

        **Parameters**:
        - `timeout`: Amount of time to wait (in seconds) for the statistics report to become available.
        """
        features = self._get_features()
        if "COMPUTE_STATISTICS" not in features:
            raise Exception("DataLab does not support computing statistics")

        data_lab = self.client.get_data_lab(self.data_lab_id)
        if data_lab["statistics"]:
            # If we already have the statistics, return them.
            return data_lab["statistics"]

        statistics_compute_job_id = data_lab["statisticsComputeJobId"]
        if not statistics_compute_job_id:
            raise Exception(
                "Statistics compute job ID not found. Please run the DataLab."
            )

        driver_attestation_hash = data_lab["jobsDriverAttestationHash"]
        if not driver_attestation_hash:
            raise Exception("Driver attestation spec not found")

        driver_spec = self.client._get_enclave_spec_from_hash(driver_attestation_hash)
        if not driver_spec:
            raise Exception(f"Failed to find driver for data lab {self.data_lab_id}")

        session = create_session_from_driver_spec(self.client, driver_spec)
        job_id = JobId(statistics_compute_job_id, "publisher_data_statistics")
        results = session.get_computation_result(job_id, interval=5, timeout=timeout)
        zip = zipfile.ZipFile(io.BytesIO(results), "r")
        if "statistics.json" in zip.namelist():
            statistics = zip.read("statistics.json").decode()
            # Store the statistics.
            self.client._set_datalab_statistics(self.data_lab_id, statistics)
            report = json.loads(statistics)
            return report
        else:
            raise Exception("Failed to retrieve statistics")

    def provision_to_lookalike_media_data_room(
        self, data_room_id: str, keychain: Keychain
    ):
        """
        Provision the DataLab to the DCR with the given ID.

        **Parameters**:
        - `data_room_id`: ID of the DCR to provision the DataLab to.
        - `keychain`: Keychain to use to provision the datasets.
        """
        # DataLab must be validated before it can be provisioned.
        if not self._validated():
            # Retrieve and store the statistics. This will validate the DataLab.
            self.get_statistics_report()
            # Check again if the DataLab is validated.
            if not self._validated():
                raise Exception("Can't provision to DCR. DataLab not validated.")

        # Check DataLab and LMDCR are compatible.
        lmdcr, lmdcr_session = self._get_lmdcr(data_room_id)
        compatible = compiler.is_data_lab_compatible_with_lookalike_media_data_room(
            self.hl_data_lab, lmdcr
        )
        if not compatible:
            raise Exception("DataLab is incompatible with DCR")

        lmdcr_datasets = compiler.get_consumed_datasets(lmdcr)
        data_lab = self.client.get_data_lab(self.data_lab_id)
        data_lab_datasets = self._get_data_lab_datasets_dict(data_lab)
        # Check all required datasets can be provisioned by the DataLab before
        # actually provisioning. This is necessary because we could otherwise
        # provision only some of the datasets resulting in a "broken" LMDCR.
        for required_dataset in lmdcr_datasets.required:
            if not data_lab_datasets[required_dataset]["manifestHash"]:
                raise Exception(
                    f"DataLab does not provide the required dataset {required_dataset}"
                )

        # Provision all required datasets.
        for required_dataset in lmdcr_datasets.required:
            lmdcr_node_name = self._get_lmdcr_node_name(required_dataset)
            manifest_hash = data_lab_datasets[required_dataset]["manifestHash"]
            retrieved_key = keychain.get("dataset_key", manifest_hash)
            lmdcr_session.publish_dataset(
                data_room_id, manifest_hash, lmdcr_node_name, Key(retrieved_key.value)
            )

        # Provision optional datasets if the DataLab is able to.
        for optional_dataset in lmdcr_datasets.optional:
            if (
                optional_dataset not in data_lab_datasets
                or not data_lab_datasets[optional_dataset]
            ):
                # If we can't provision the dataset, move on to the next.
                continue
            lmdcr_node_name = self._get_lmdcr_node_name(optional_dataset)
            manifest_hash = data_lab_datasets[optional_dataset]["manifestHash"]
            retrieved_key = keychain.get("dataset_key", manifest_hash)
            lmdcr_session.publish_dataset(
                data_room_id, manifest_hash, lmdcr_node_name, Key(retrieved_key.value)
            )

    @staticmethod
    def is_validation_passed(validation_report: Dict[str, str]) -> bool:
        return (
            validation_report["dataset_users"]["report"]["outcome"] == "PASSED"
            and validation_report["dataset_segments"]["report"]["outcome"] == "PASSED"
            and validation_report["dataset_embeddings"]["report"]["outcome"] == "PASSED"
            and validation_report["dataset_demographics"]["report"]["outcome"]
            == "PASSED"
        )

    def _validated(self) -> bool:
        data_lab = self.client.get_data_lab(self.data_lab_id)
        return data_lab["isValidated"]

    def _get_data_lab_datasets_dict(self, data_lab: compiler.DataLab):
        datasets_dict = {}
        for dataset in data_lab["datasets"]:
            datasets_dict[dataset["name"]] = dataset["dataset"]
        return datasets_dict

    def _get_lmdcr(self, data_room_id) -> (LookalikeMediaDataRoom, Session):
        # Get the high level representation of the LMDCR.
        lmdcr_driver_attestation_hash = self.client._get_lmdcr_driver_attestation_hash(
            data_room_id
        )
        lmdcr_driver_spec = self.client._get_enclave_spec_from_hash(
            lmdcr_driver_attestation_hash
        )
        if lmdcr_driver_spec == None:
            raise Exception(f"Failed to find driver for data room {data_room_id}")

        session = create_session_from_driver_spec(self.client, lmdcr_driver_spec)
        existing_lmdcr = session.retrieve_data_room(data_room_id)
        lmdcr_hl = json.loads(existing_lmdcr.highLevelRepresentation.decode())
        lmdcr = LookalikeMediaDataRoom.parse_obj(lmdcr_hl)

        # Verify data room.
        # Verification involves compiling the LMDCR again and checking that the
        # low level representation matches the original low level representation.
        compiled_serialized = compiler.compile_lookalike_media_data_room(lmdcr)
        recompiled_low_level_dcr = DataRoom()
        parse_length_delimited(compiled_serialized, recompiled_low_level_dcr)
        if recompiled_low_level_dcr != existing_lmdcr.dataRoom:
            raise Exception("LMDCR failed verification")
        return (lmdcr, session)

    # Construct the actual DCR that implements the DataLab functionality.
    def _construct_backing_dcr(self, session: Session) -> str:
        compiled_serialized = compiler.compile_data_lab(self.hl_data_lab)
        low_level_dcr = DataRoom()
        parse_length_delimited(compiled_serialized, low_level_dcr)
        # By default the backing DCR will have the same ID as the DataLab.
        # This causes an issue if there has previously been a DCR created for the DataLab. Instead generate a new ID.
        low_level_dcr.id = str(uuid4())
        dcr_id = session.publish_data_room(
            low_level_dcr, purpose=CreateDcrPurpose.DATA_LAB
        )
        return dcr_id

    # Updates the HL DataLab representation and the session object.
    def _update_enclave_specs(self):
        enclave_specs = get_latest_enclave_specs_as_dictionary(self.client)
        (driver_spec, python_spec) = self._get_data_lab_enclave_specs(enclave_specs)
        # Update to the latest enclave specs
        root_certificate_pem = self.client.decentriq_ca_root_certificate.decode("utf-8")
        # Update the HL representation with the new enclave specs.
        self.hl_data_lab = compiler.update_data_lab_enclave_specifications(
            self.hl_data_lab, driver_spec, python_spec, root_certificate_pem
        )
        auth, _ = self.client.create_auth_using_decentriq_pki(enclave_specs)
        self.session = self.client.create_session(auth, enclave_specs)

    def _get_data_lab_node_names(self, dataset_type: DataLabDatasetType):
        if dataset_type == DataLabDatasetType.EMBEDDINGS:
            return compiler.get_data_lab_node_id(compiler.DataLabNode.Embeddings)
        elif dataset_type == DataLabDatasetType.DEMOGRAPHICS:
            return compiler.get_data_lab_node_id(compiler.DataLabNode.Demographics)
        elif dataset_type == DataLabDatasetType.MATCH:
            return compiler.get_data_lab_node_id(compiler.DataLabNode.Users)
        elif dataset_type == DataLabDatasetType.SEGMENTS:
            return compiler.get_data_lab_node_id(compiler.DataLabNode.Segments)

    def _get_validation_nodes(self):
        features = self._get_features()
        validation_nodes = []
        if "VALIDATE_MATCHING" in features:
            users = self._get_data_lab_node_names(DataLabDatasetType.MATCH)
            validation_nodes.append(users)
        if "VALIDATE_SEGMENTS" in features:
            segments = self._get_data_lab_node_names(DataLabDatasetType.SEGMENTS)
            validation_nodes.append(segments)
        if "VALIDATE_EMBEDDINGS" in features and self.cfg.has_embeddings:
            embeddings = self._get_data_lab_node_names(DataLabDatasetType.EMBEDDINGS)
            validation_nodes.append(embeddings)
        if "VALIDATE_DEMOGRAPHICS" in features and self.cfg.has_demographics:
            demographics = self._get_data_lab_node_names(
                DataLabDatasetType.DEMOGRAPHICS
            )
            validation_nodes.append(demographics)
        # Add the appropriate suffix for the validation nodes.
        validation_nodes = [node + "_validation_report" for node in validation_nodes]
        return validation_nodes

    def _get_features(self):
        features = compiler.get_data_lab_features(self.hl_data_lab)
        return features

    def _get_lmdcr_node_name(self, dataset_name: str):
        lmdcr_node_name = (
            compiler.get_lookalike_media_node_names_from_data_lab_data_type(
                dataset_name
            )
        )
        if not lmdcr_node_name:
            raise Exception(f"Unknown LMDCR node name for dataset name {dataset_name}")
        return lmdcr_node_name
